(************************************************************************
 Author:	Robert Rossmair
		Wagelwitz, Rodaer Str. 6
		04688 Mutzschen
		Germany

	e-mail:	Robert.Rossmair@t-online.de

 Module:	RRUtils

 Version:	10-May-1999

 Description:	general useful(?) declarations and routines

 Copyright  1997-1999  Robert Rossmair
 ************************************************************************)
unit RRUtils;

interface

uses SysUtils, Windows, Classes, Controls, Forms;

type
  MilliSec = Longint;

  ArrayIndex = 0..99999999;	// 0..10**8-1

  PSmallint = ^Smallint;
  TSmallintArray = array[ArrayIndex] of Smallint;	PSmallintArray = ^TSmallintArray;
  TIntArray = array[ArrayIndex] of Integer;		PIntArray = ^TIntArray;
  TLongArray = array[ArrayIndex] of Longint;		PLongArray = ^TLongArray;

const
  PowerOn  = true;
  PowerOff = not PowerOn;
  Invalid	= -1;

  BitsPerByte = 8;
  BitsPerWord = SizeOf(Word) * BitsPerByte;
  BitsPerDWord = SizeOf(Longint) * BitsPerByte;


  MinutesPerHour= 60;
  SecondsPerMin	= 60;
  SecondsPerHour= MinutesPerHour * SecondsPerMin;

  Percent	= 0.01;

  dB = 0.0029397460514;		{ 96 dB / 32768 }

function BaseFileName(FileName: string): string;
function Equal(const X, Y; VarSize: DWord): boolean; register;
procedure MouseBusy;
procedure MouseLazy;
function ReadPoint(S: TStream): TPoint;
function ReadRect(S: TStream): TRect;
function ReadSmallInt(S: TStream): SmallInt;
function ReadLongInt(S: TStream): LongInt;
function ReadPtr(S: TStream): Pointer;
function StrRead(S: TStream): PChar;
procedure StrWrite(S: TStream; P: PChar);
procedure WritePoint(S: TStream; const P: TPoint);
procedure WriteRect(S: TStream; const R: TRect);
procedure WriteSmallInt(S: TStream; Value: SmallInt);
procedure WriteLongInt(S: TStream; Value: LongInt);
function StartTimer(var BaseTime: MilliSec): MilliSec;
function GetTimer(var BaseTime: MilliSec): MilliSec;
function UniqueFileName(BaseName, Ext: string): string;
procedure Xchange(var a, b; Size: DWord); register;

implementation

function BaseFileName(FileName: string): string;
var
  I: Integer;
begin
  Result := ExtractFileName(FileName);
  I := Length(Result);
  for I := I downto I-3 do
  if Result[I] = '.' then
    Result := Copy(Result, 1, I-1);
end;

function Equal(const X, Y; VarSize: DWord): boolean;
var
  I: Integer;
begin
  Result := False;
  for I := 0 to VarSize-1 do
    if TByteArray(X)[I] <> TByteArray(Y)[I] then Exit;
  Result := True;
end;

const
  Busy: Integer = 0;

procedure MouseBusy;
begin
  if Busy = 0 then
    Screen.Cursor := crHourglass;
  Inc(Busy);
end;

procedure MouseLazy;
begin
  if Busy > 0 then
  begin
    Dec(Busy);
    if Busy = 0 then Screen.Cursor := crDefault;
  end;
end;

function ReadRect(S: TStream): TRect;
begin
  S.Read(Result, SizeOf(Result));
end;

function ReadSmallInt(S: TStream): SmallInt;
begin
  S.Read(Result, SizeOf(Result));
end;

function ReadLongInt(S: TStream): LongInt;
begin
  S.Read(Result, SizeOf(Result));
end;

function ReadPoint(S: TStream): TPoint;
begin
  S.Read(Result, SizeOf(Result));
end;

function ReadPtr(S: TStream): Pointer;
begin
  S.Read(Result, SizeOf(Result));
end;

function StrRead(S: TStream): PChar;
var
  L: Word;
begin
  S.Read(L, SizeOf(L));
  if L = 0 then Result := nil else
  begin
    Result := StrAlloc(L+1);
    S.Read(Result^, L);
    Result[L] := #0;	{ sicher ist sicher }
  end;
end;

procedure StrWrite(S: TStream; P: PChar);
var
  L: Word;
begin
  if P = nil then L := 0 else L := StrLen(P);
  S.Write(L, SizeOf(Word));
  if P <> nil then S.Write(P[0], L);
end;

procedure WritePoint(S: TStream; const P: TPoint);
begin
  S.Write(P, SizeOf(P));
end;

procedure WriteRect(S: TStream; const R: TRect);
begin
  S.Write(R, SizeOf(R));
end;

procedure WriteSmallInt(S: TStream; Value: SmallInt);
begin
  S.Write(Value, SizeOf(Value));
end;

procedure WriteLongInt(S: TStream; Value: LongInt);
begin
  S.Write(Value, SizeOf(Value));
end;

function StartTimer(var BaseTime: MilliSec): MilliSec;
begin
  Result := GetTickCount;
  BaseTime := Result;
end;

function GetTimer(var BaseTime: MilliSec): MilliSec;
begin
  Result := GetTickCount - BaseTime;
end;

function UniqueFileName(BaseName, Ext: string): string;
var
  NameNum: Integer;
begin
  NameNum := 0;
  Result := BaseName+Ext;
  while FileExists(Result) do
  begin
    Result := Format('%s%.3d%s', [BaseName, NameNum, Ext]);
    Inc(NameNum);
  end;
end;

procedure Xchange(var a, b; Size: DWord); assembler;
asm
	push	esi
	push	edi
	mov	esi, a
	mov	edi, b
	cld
	mov	edx, ecx
	shr	ecx, 2
	jz	@1
@loop:	mov	eax, [edi]
	movsd
	mov	[esi-4], eax
	loop	@loop
@1:     and	dl, 3
	jz	@exit
@2:	mov	al, [edi]
	movsb
	mov	[esi-1], al
	dec	dl
	jnz	@2
@exit:	pop	edi
	pop	esi
end;

end.
